<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author   SoftBuild <dev@soft-build.by>
 * @copyright 2017-2022 SoftBuild
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

use Gratify\PhpSdk\Client as GratifyClient;
use Gratify\PhpSdk\Request\OrderInitialize;
use Gratify\PhpSdk\Request\ShippingAddress;
use Gratify\PhpSdk\Request\Item;

class Gratify_PrestashopRedirectModuleFrontController extends ModuleFrontController
{
    /**
     * Do whatever you have to before redirecting the customer on the website of your payment processor.
     */
    public function postProcess()
    {
        try {
            $cart = Context::getContext()->cart;

            $client = new GratifyClient(
                Configuration::get('GRATIFY_MERCHANT_ID'),
                Configuration::get('GRATIFY_SECRET_KEY'),
                Configuration::get('GRATIFY_LIVE_MODE'),
                AddressHelper::isUs($cart->id_address_delivery)
            );

            $deliveryAddress = new Address($cart->id_address_delivery);
            $country = new Country($deliveryAddress->id_country);
            $state = new State($deliveryAddress->id_state);
            $shippingAddress = new ShippingAddress();
            $shippingAddress->setCity($deliveryAddress->city)
                ->setCountry($country->iso_code)
                ->setLine1($deliveryAddress->address1)
                ->setLine2($deliveryAddress->address2)
                ->setState($state->iso_code ? $state->iso_code : '')
                ->setPostcode($deliveryAddress->postcode);

            $queryParams = [
                'cart_id' => $cart->id,
                'secure_key' => Context::getContext()->customer->secure_key,
            ];
            $order = new OrderInitialize();
            $order->setStorePlatform('PrestaShop: ' . _PS_VERSION_)
                ->setReturnURL(
                    $this->context->link->getModuleLink(
                        $this->module->name,
                        'success',
                        $queryParams,
                        true
                    )
                )
                ->setCallbackURL($this->context->link->getModuleLink(
                    $this->module->name,
                    'callback',
                    $queryParams,
                    true
                ))
                ->setFailureURL($this->context->link->getModuleLink(
                    $this->module->name,
                    'failed',
                    $queryParams,
                    true
                ))
                ->setShippingMethod(OrderInitialize::SHIPPING_METHOD)
                ->setMerchantReference($cart->id)
                ->setStorePlatform(Gratify_Prestashop::STORE_PLATFORM)
                ->setShippingAddress($shippingAddress)
                ->setPriceSubTotal(Tools::ps_round($cart->getOrderTotal(false, Cart::ONLY_PRODUCTS), Context::getContext()->getComputingPrecision()))
                ->setShippingTotal(Tools::ps_round($cart->getOrderTotal(false, Cart::ONLY_SHIPPING), Context::getContext()->getComputingPrecision()))
                ->setTaxesTotal(Tools::ps_round($cart->getOrderTotal() - $cart->getOrderTotal(false), Context::getContext()->getComputingPrecision()));

            foreach ($cart->getProducts() as $product) {
                $imageId = '';
                if (!$images = Image::getImages(Context::getContext()->language->id, $product['id_product'], $product['id_product_attribute'])) {
                    $images = Image::getImages(Context::getContext()->language->id, $product['id_product']);
                }

                if ($images) {
                    foreach ($images as $image) {
                        if ($image['cover']) {
                            $imageId = $image['id_image'] ?? '';
                        }
                    }

                    if (empty($imageId)) {
                        $imageId = $images[0]['id_image'] ?? '';
                    }
                }
                $item = new Item();
                $item->setId($product['id_product'])
                    ->setPrice(Tools::ps_round($product['price'], Context::getContext()->getComputingPrecision()))
                    ->setTotalPrice(Tools::ps_round($product['cart_quantity'] * $product['price'], Context::getContext()->getComputingPrecision()))
                    ->setQuantity((int)$product['cart_quantity'])
                    ->setDescription(strip_tags($product['description_short']))
                    ->setImageURL(
                        Context::getContext()->link->getImageLink(
                            $product['link_rewrite'],
                            $imageId,
                            'large_default'
                        )
                    )
                    ->setTitle($product['name']);
                $order->setItem($item);
            }

            if ($initializedOrder = $client->orderInitialize($order)) {
                $gratifyOrder = new GratifyOrder();
                $gratifyOrder->id_cart = $cart->id;
                $gratifyOrder->id_gratify_map_order = $initializedOrder->getOrderMapId();
                $gratifyOrder->id_gratify_order = $initializedOrder->getOrderId();
                $gratifyOrder->save();

                $redirectLink = $client->getRedirectLink($initializedOrder->getOrderMapId());
                Tools::redirect($redirectLink);
            }
        } catch (\Exception $e) {
            $message = sprintf('GratifyPay: %s, issue reference: %s', $e->getMessage(), $cart->id);
            PrestaShopLogger::addLog(
                $message,
                3,
                null,
                'GratifyPay'
            );
        }

        $this->context->smarty->assign(array(
            'referenceIssue' => $cart->id,
        ));

        return $this->setTemplate('module:gratify/views/templates/front/redirect.tpl');
    }
}
