<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author   Gratify Pay <support@gratifypay.com>
 * @copyright 2021-2022 Gratify Pay
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

use Gratify\PhpSdk\Request\RefundItem;
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once dirname(__FILE__) . '/include.php';

use Gratify\PhpSdk\Client as GratifyClient;
use Gratify\PhpSdk\Request\Refund;

class Gratify_Prestashop extends PaymentModule
{
    protected $config_form = false;

    const STORE_PLATFORM = 'PRESTASHOP';

    public function __construct()
    {
        $this->name = 'gratify_prestashop';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.0';
        $this->author = 'Gratify Pay';
        $this->need_instance = 0;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Gratify BNPL');
        $this->description = $this->l(
            'Sell more with Gratify Buy Now Pay Later'
        );

        $this->limited_countries = array('US', 'CA', 'AU', 'NZ');

        $this->limited_currencies = array('USD', 'CAD', 'AUD', 'NZD');

        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }

        require_once dirname(__FILE__) . '/sql/install.php';

        $iso_code = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));

        if (in_array($iso_code, $this->limited_countries) == false) {
            $this->_errors[] = $this->l('This module is not available in your country');
            return false;
        }

        Configuration::updateValue('GRATIFY_LIVE_MODE', false);
        Configuration::updateValue('GRATIFY_CATEGORY_PAGE', true);
        Configuration::updateValue('GRATIFY_PRODUCT_PAGE', true);
        Configuration::updateValue('GRATIFY_CART_PAGE', true);
        Configuration::updateValue('GRATIFY_INDEX_PAGE', true);

        $gratifyPaymentPhrase = 'or {NUM_PAYMENTS} interest-free instalments of {AMOUNT} by {GRATIFY_LOGO}';
        Configuration::updateValue('GRATIFY_CATEGORY_PAGE_PHRASE', $gratifyPaymentPhrase);
        Configuration::updateValue('GRATIFY_PRODUCT_PAGE_PHRASE', $gratifyPaymentPhrase);
        Configuration::updateValue('GRATIFY_CART_PAGE_PHRASE', $gratifyPaymentPhrase);
        Configuration::updateValue('GRATIFY_INDEX_PAGE_PHRASE', $gratifyPaymentPhrase);

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
            $this->registerHook('displayRightColumnProduct') &&
            $this->registerHook('displayLeftColumnProduct') &&
            $this->registerHook('displayProductPriceBlock') &&
            $this->registerHook('actionObjectOrderSlipAddAfter') &&
            $this->registerHook('actionOrderSlipAdd') &&
            $this->registerHook('displayProductListReviews') &&
            $this->registerHook('displayReassurance') &&
            $this->registerHook('paymentOptions');
    }

    public function uninstall()
    {
        require_once dirname(__FILE__) . '/sql/uninstall.php';

        Configuration::deleteByName('GRATIFY_LIVE_MODE');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitGratifyModule')) == true) {
            $this->postProcess();
        }

        $this->context->smarty->assign('module_dir', $this->_path);

        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');

        return $this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitGratifyModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    /*array(
                        'type' => 'switch',
                        'label' => $this->l('Live mode'),
                        'name' => 'GRATIFY_LIVE_MODE',
                        'is_bool' => true,
                        'desc' => $this->l('Use this module in live mode'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Yes')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('No')
                            )
                        ),
                    ),*/
                    array(
                        'col' => 6,
                        'type' => 'text',
                        'desc' => $this->l('Enter a merchant id'),
                        'name' => 'GRATIFY_MERCHANT_ID',
                        'label' => $this->l('Merchant ID'),
                    ),
                    array(
                        'col' => 6,
                        'type' => 'text',
                        'desc' => $this->l('Enter a secret key'),
                        'name' => 'GRATIFY_SECRET_KEY',
                        'label' => $this->l('Secret Key'),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Home Page'),
                        'name' => 'GRATIFY_INDEX_PAGE',
                        'is_bool' => true,
                        'desc' => $this->l('Show the phrase at a home page'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Yes')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('No')
                            )
                        ),
                    ),
                    array(
                        'col' => 6,
                        'type' => 'textarea',
                        'label' => $this->l('Home page customization'),
                        'name' => 'GRATIFY_INDEX_PAGE_PHRASE',
                        'autoload_rte' => true,
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Category Page'),
                        'name' => 'GRATIFY_CATEGORY_PAGE',
                        'is_bool' => true,
                        'desc' => $this->l('Show the phrase at a category page'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Yes')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('No')
                            )
                        ),
                    ),
                    array(
                        'col' => 6,
                        'type' => 'textarea',
                        'label' => $this->l('Category page customization'),
                        'name' => 'GRATIFY_CATEGORY_PAGE_PHRASE',
                        'autoload_rte' => true,
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Product Page'),
                        'name' => 'GRATIFY_PRODUCT_PAGE',
                        'is_bool' => true,
                        'desc' => $this->l('Show the phrase at a product page'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Yes')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('No')
                            )
                        ),
                    ),
                    array(
                        'col' => 6,
                        'type' => 'textarea',
                        'label' => $this->l('Product page customization'),
                        'name' => 'GRATIFY_PRODUCT_PAGE_PHRASE',
                        'autoload_rte' => true,
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Cart Page'),
                        'name' => 'GRATIFY_CART_PAGE',
                        'is_bool' => true,
                        'desc' => $this->l('Show the phrase at a cart page'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Yes')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('No')
                            )
                        ),
                    ),
                    array(
                        'col' => 6,
                        'type' => 'textarea',
                        'label' => $this->l('Cart page customization'),
                        'name' => 'GRATIFY_CART_PAGE_PHRASE',
                        'autoload_rte' => true,
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'GRATIFY_LIVE_MODE' => Configuration::get('GRATIFY_LIVE_MODE'),
            'GRATIFY_MERCHANT_ID' => Configuration::get('GRATIFY_MERCHANT_ID'),
            'GRATIFY_SECRET_KEY' => Configuration::get('GRATIFY_SECRET_KEY'),
            'GRATIFY_CATEGORY_PAGE' => Configuration::get('GRATIFY_CATEGORY_PAGE'),
            'GRATIFY_CATEGORY_PAGE_PHRASE' => Configuration::get('GRATIFY_CATEGORY_PAGE_PHRASE'),
            'GRATIFY_PRODUCT_PAGE' => Configuration::get('GRATIFY_PRODUCT_PAGE'),
            'GRATIFY_PRODUCT_PAGE_PHRASE' => Configuration::get('GRATIFY_PRODUCT_PAGE_PHRASE'),
            'GRATIFY_CART_PAGE' => Configuration::get('GRATIFY_CART_PAGE'),
            'GRATIFY_CART_PAGE_PHRASE' => Configuration::get('GRATIFY_CART_PAGE_PHRASE'),
            'GRATIFY_INDEX_PAGE' => Configuration::get('GRATIFY_INDEX_PAGE'),
            'GRATIFY_INDEX_PAGE_PHRASE' => Configuration::get('GRATIFY_INDEX_PAGE_PHRASE'),
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key), true);
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader($params)
    {
        $totalAmount = Context::getContext()->cart->getOrderTotal();
        $key = 'gratify_checkout_' . $totalAmount;
        if ($this->context->controller->php_self == 'order' && !Cache::isStored($key)) {
            $client = new GratifyClient(
                Configuration::get('GRATIFY_MERCHANT_ID'),
                Configuration::get('GRATIFY_SECRET_KEY'),
                Configuration::get('GRATIFY_LIVE_MODE'),
                AddressHelper::isUs(Context::getContext()->cart->id_address_delivery)
            );

            try {
                $merchant = $client->getMerchant();
            } catch (\Exception $e) {
                PrestaShopLogger::addLog(sprintf("Gratify Pay exception: %s", $e->getMessage()));
                return;
            }
            if ($totalAmount >= $merchant->getMinAmount() && $totalAmount <= $merchant->getMaxAmount()) {
                $this->context->controller->registerJavascript(
                    'gratify-schedule-app',
                    'https://assets.gratifypay.com/plugin/v-latest/widget.js?ver=1.0.1',
                    ['server' => 'remote', 'position' => 'bottom', 'priority' => 20]
                );
                $this->context->controller->registerStylesheet(
                    'gratify-schedule-qp',
                    'https://assets.gratifypay.com/plugin/v-latest/styles.css?ver=1.0.1',
                    ['server' => 'remote', 'position' => 'bottom', 'priority' => 20]
                );

                $paymentSchedule = $client->getPaymentsSchedule($params['cart']->getOrderTotal(), true);

                $schedules = [];
                foreach ($paymentSchedule->getSchedules() as $schedule) {
                    $schedules[] = [
                        'date' => $schedule->getDate(),
                        'amount' => $schedule->getAmount(),
                        'dueFromNow' => $schedule->getDueFromNow(),
                        'paymentNumber' => $schedule->getPaymentNumber(),
                        'totalAmount' => $schedule->getTotalAmount(),
                    ];
                }

                Cache::store($key, [
                    'gratifyPaySchedules' => $schedules,
                    'gratifyPayMeta' => [
                        'numPayments' => $paymentSchedule->getMeta()->getNumPayments(),
                        'totalAmount' => $paymentSchedule->getMeta()->getTotalAmount(),
                    ],
                ]);
            }
        }

        Media::addJsDef(Cache::retrieve($key));

        $this->context->controller->addJS($this->_path . 'views/js/front.js');
        $this->context->controller->addCSS($this->_path . '/views/css/front.css');
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $option = null;
        $totalAmount = $params['cart']->getOrderTotal();
        $key = 'gratify_card_' . $totalAmount;
        if (!Cache::isStored($key)) {
            $client = new GratifyClient(
                Configuration::get('GRATIFY_MERCHANT_ID'),
                Configuration::get('GRATIFY_SECRET_KEY'),
                Configuration::get('GRATIFY_LIVE_MODE'),
                AddressHelper::isUs($params['cart']->id_address_delivery)
            );

            try {
                $merchant = $client->getMerchant();
            } catch (\Exception $e) {
                PrestaShopLogger::addLog(sprintf("Gratify Pay exception: %s", $e->getMessage()));
                return;
            }
            if ($totalAmount >= $merchant->getMinAmount() && $totalAmount <= $merchant->getMaxAmount()) {
                $option = new PaymentOption();
                $option->setCallToActionText($this->l($this->displayName))
                    ->setModuleName($this->name)
                    ->setLogo('https://assets.gratifypay.com/img/gratify-badge-blue.svg')
                    ->setAdditionalInformation($this->context->smarty->fetch('module:gratify/views/templates/front/payment_infos.tpl'))
                    ->setAction($this->context->link->getModuleLink($this->name, 'redirect', array(), true));

                Cache::store($key, $option);
            }
        }
        if (!$option = Cache::retrieve($key)) {
            return;
        }

        return [
            $option
        ];
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function getGratifyPhrase($controller, $numPayments, $amount)
    {
        $search = ['{NUM_PAYMENTS}', '{AMOUNT}', '{GRATIFY_LOGO}'];
        $replace = [
            $numPayments,
            $amount,
            '<img style="margin-top:-3px;vertical-align:middle;display:inline-block;border:none;"
            src="https://assets.gratifypay.com/img/gratify-badge-blue.svg" width="80" height="30" alt="Gratify Pay" />'
        ];
        $phrase = '';
        switch ($controller) {
            case 'category':
                $phrase = Configuration::get('GRATIFY_CATEGORY_PAGE_PHRASE');
                break;
            case 'product':
                $phrase = Configuration::get('GRATIFY_PRODUCT_PAGE_PHRASE');
                break;
            case 'cart':
                $phrase = Configuration::get('GRATIFY_CART_PAGE_PHRASE');
                break;
            case 'index':
                $phrase = Configuration::get('GRATIFY_INDEX_PAGE_PHRASE');
                break;
        }

        return str_replace($search, $replace, $phrase);
    }

    public function isShow()
    {
        switch ($this->context->controller->php_self) {
            case 'category':
                if (Configuration::get('GRATIFY_CATEGORY_PAGE')) {
                    return true;
                }
                break;
            case 'product':
                if (Configuration::get('GRATIFY_PRODUCT_PAGE')) {
                    return true;
                }
                break;
            case 'cart':
                if (Configuration::get('GRATIFY_CART_PAGE')) {
                    return true;
                }
                break;
            case 'index':
                if (Configuration::get('GRATIFY_INDEX_PAGE')) {
                    return true;
                }
                break;
        }

        return false;
    }

    public function getScheduleBlock($price)
    {
        if (!$this->active) {
            return;
        }

        if (!$this->isShow()) {
            return;
        }

        $key = 'product_gratify_' . $price;
        if (!Cache::isStored($key)) {
            $client = new GratifyClient(
                Configuration::get('GRATIFY_MERCHANT_ID'),
                Configuration::get('GRATIFY_SECRET_KEY'),
                Configuration::get('GRATIFY_LIVE_MODE'),
                AddressHelper::isUs(Context::getContext()->cart->id_address_delivery)
            );

            $result = '';
            try {
                $merchant = $client->getMerchant();
            } catch (\Exception $e) {
                PrestaShopLogger::addLog(sprintf("Gratify Pay exception: %s", $e->getMessage()));
                return;
            }
            if ($price >= $merchant->getMinAmount() && $price <= $merchant->getMaxAmount()) {
                $paymentSchedule = $client->getPaymentsSchedule($price);

                $amount = 0;
                $countPayments = 0;
                if ($schedules = $paymentSchedule->getSchedules()) {
                    $isoCode = $this->context->currency->iso_code;
                    $amount = $schedules[0] ? $this->context->getCurrentLocale()->formatPrice($schedules[0]->getAmount(), $isoCode) : 0;
                    $countPayments = count($schedules);
                }
                $this->context->smarty->assign([
                    'amount' => $amount,
                    'countPayments' => $countPayments,
                    'phrase' => $this->getGratifyPhrase($this->context->controller->php_self, $countPayments, $amount),
                ]);

                $result = $this->context->smarty->fetch('module:gratify/views/templates/front/payment_period.tpl');
            }

            Cache::store($key, $result);
        }

        return Cache::retrieve($key);
    }

    public function hookDisplayReassurance($params)
    {
        if ($this->context->controller->php_self == 'cart') {
            return $this->getScheduleBlock(Context::getContext()->cart->getOrderTotal());
        }
    }

    public function hookDisplayProductListReviews($params)
    {
        $price = Product::getPriceStatic($params['product']['id'], true, $params['product']['id_product_attribute']);
        return $this->getScheduleBlock($price);
    }

    public function hookDisplayProductPriceBlock($params)
    {
        if ($params['type'] == 'after_price') {
            $price = Product::getPriceStatic($params['product']['id'], true, $params['product']['id_product_attribute']);
            return $this->getScheduleBlock($price);
        }
    }

    public static function getMerchantRefundId()
    {
        return md5(rand(1, 100000));
    }

    public function hookActionOrderSlipAdd($params)
    {
        //todo partially refund does not created yet in gratify pay
        return;

        /**
         * @var Order $order
         */
        $order = $params['order'];
        if (!empty($params['productList'])) {

            /**
             * @var GratifyOrder $gratifyOrder
             */
            if ($gratifyOrder = GratifyOrder::getByOrderId($order->id)) {
                try {
                    $client = new GratifyClient(
                        Configuration::get('GRATIFY_MERCHANT_ID'),
                        Configuration::get('GRATIFY_SECRET_KEY'),
                        Configuration::get('GRATIFY_LIVE_MODE'),
                        AddressHelper::isUs($order->id_address_delivery)
                    );

                    $refund = new Refund();
                    $totalRefundPrice = 0;
                    foreach ($params['productList'] as $row) {
                        $productDetail = new OrderDetail($row['id_order_detail']);
                        $product = new Product($productDetail->product_id);

                        $item = new RefundItem();
                        $item->setId($product->id)
                            ->setSku($product->reference)
                            ->setQty($row['quantity'])
                            ->setPrice($row['amount']);
                        $totalPrice = $row['quantity'] * $row['amount'];
                        $item->setTotalPrice($totalPrice);
                        $refund->setRefundItems($item);

                        $totalRefundPrice += $totalPrice;
                    }

                    $merchantRefundId = self::getMerchantRefundId();

                    $refund->setMerchantRefundId($merchantRefundId)
                        ->setOrderId($gratifyOrder->id_gratify_order)
                        ->setStorePlatform(self::STORE_PLATFORM)
                        ->setCurrency(Currency::getIsoCodeById($order->id_currency))
                        ->setAmount($totalRefundPrice);
                    $refundResponse = $client->orderRefund($refund);

                    /*switch ($refundResponse->getRefundStatus()) {
                        case \Gratify\PhpSdk\Response\Refund::REFUND_REFUNDED:
                            $msg = new Message();
                            $msg->message = sprintf(
                                "GratifyPay: for the order: %s, product amount: %d success refunded",
                                $order->id,
                                $totalRefundPrice
                            );
                            $msg->id_order = intval($order->id);
                            $msg->private = 1;
                            $msg->add();
                            break;
                        case \Gratify\PhpSdk\Response\Refund::REFUND_DECLINED:
                        case \Gratify\PhpSdk\Response\Refund::REFUND_ERROR:
                            throw new Exception($refundResponse->getMessage());
                            break;
                    }*/

                    if ($refundResponse->isSuccess()) {
                        $customer = new Customer($order->id_customer);
                        $message = sprintf(
                            "GratifyPay: %s, for amount: %s",
                            $refundResponse->getMessage(),
                            $orderSlip->amount
                        );
                        $this->saveCustomerMessage($order, $customer, $message);

                        /*$order->addOrderPayment($orderSlip->amount, $this->displayName, $merchantRefundId);*/
                    } else {
                        throw new Exception($refundResponse->getMessage());
                    }
                } catch (\Exception $e) {
                    $message = sprintf('GratifyPay: %s, order id: %s', $e->getMessage(), $order->id);
                    PrestaShopLogger::addLog(
                        $message,
                        3,
                        null,
                        'GratifyPay'
                    );
                }
            }
        }
    }

    public function hookActionObjectOrderSlipAddAfter($params)
    {
        /**
         * @var OrderSlip $orderSlip
         */
        $orderSlip = $params['object'];
        /**
         * @var GratifyOrder $gratifyOrder
         */
        if ($gratifyOrder = GratifyOrder::getByOrderId($orderSlip->id_order)) {
            $order = new Order($orderSlip->id_order);

            try {
                $client = new GratifyClient(
                    Configuration::get('GRATIFY_MERCHANT_ID'),
                    Configuration::get('GRATIFY_SECRET_KEY'),
                    Configuration::get('GRATIFY_LIVE_MODE'),
                    AddressHelper::isUs($order->id_address_delivery)
                );
                $refund = new Refund();
                $merchantRefundId = self::getMerchantRefundId();
                $refund->setMerchantRefundId($merchantRefundId)
                    ->setOrderId($gratifyOrder->id_gratify_order)
                    ->setStorePlatform(self::STORE_PLATFORM)
                    ->setCurrency(Currency::getIsoCodeById($order->id_currency))
                    ->setAmount($orderSlip->amount);

                $refundResponse = $client->orderRefund($refund);
                if ($refundResponse->isSuccess()) {

                    $customer = new Customer($order->id_customer);
                    $message = sprintf(
                        "GratifyPay: %s, for amount: %s",
                        $refundResponse->getMessage(),
                        $orderSlip->amount
                    );
                    $this->saveCustomerMessage($order, $customer, $message);

                    /*$order->addOrderPayment($orderSlip->amount, $this->displayName, $merchantRefundId);*/
                } else {
                    throw new Exception($refundResponse->getMessage());
                }
            } catch (\Exception $e) {
                $message = sprintf('GratifyPay: %s, order id: %s', $e->getMessage(), $order->id);
                PrestaShopLogger::addLog(
                    $message,
                    3,
                    null,
                    'GratifyPay'
                );
            }
        }
    }

    /**
     * @param $order
     * @param $customer
     * @param $message
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function saveCustomerMessage($order, $customer, $message)
    {
        $msg = new Message();
        if (Validate::isCleanHtml($message)) {
            $msg->message = $message;
            $msg->id_cart = (int)$order->id_cart;
            $msg->id_order = (int)$order->id;
            $msg->private = 1;
            $msg->add();

            $customerThreadId = CustomerThread::getIdCustomerThreadByEmailAndIdOrder($customer->email, $order->id);
            if (!$customerThreadId) {
                // Add this message in the customer thread
                $customer_thread = new CustomerThread();
                $customer_thread->id_contact = 0;
                $customer_thread->id_customer = (int)$order->id_customer;
                $customer_thread->id_shop = (int)Context::getContext()->shop->id;
                $customer_thread->id_order = (int)$order->id;
                $customer_thread->id_lang = (int)Context::getContext()->language->id;
                $customer_thread->email = Context::getContext()->customer->email;
                $customer_thread->status = 'open';
                $customer_thread->token = Tools::passwdGen(12);
                $customer_thread->add();

                $customerThreadId = $customer_thread->id;
            }
            $customer_message = new CustomerMessage();
            $customer_message->id_customer_thread = $customerThreadId;
            $customer_message->id_employee = 0;
            $customer_message->message = $msg->message;
            $customer_message->private = 1;
            $customer_message->add();
        }
    }
}
